/* This is an adaptation of the example found on pg 93 of the draft */
/* This version is not draft compliant since it sends the total size */
/* of the buffer as the argument to the reduce operation. When */
/* the MAXLOC and MINLOC operations become draft compliant, we will */
/* use TABLE_SIZE instead of 2*TABLE_SIZE. */
#include "mpi.h"

#define TABLE_SIZE 30

worker( argc, argv )
int argc;
char **argv;
{
  int    rank, size;
  double a[TABLE_SIZE];
  double in[2*TABLE_SIZE], out[2*TABLE_SIZE];
  int    i, ranks[TABLE_SIZE];
  int    errors = 0;

  /* Initialize the environment and some variables */
  MPI_Init( &argc, &argv );
  MPI_Comm_rank( MPI_COMM_WORLD, &rank );
  MPI_Comm_size( MPI_COMM_WORLD, &size );

  /* Initialize the maxloc data */
  for ( i=0; i<TABLE_SIZE; i++ ) a[i] = 0;
  for ( i=rank; i<TABLE_SIZE; i+=size ) a[i] = (double)rank + 1.0;

  /* Copy data to the "in" buffer */
  for (i=0; i<TABLE_SIZE; i++) 
	in[i] = a[i];

  /* Reduce it! */
  MPI_Reduce( in, out, 2*TABLE_SIZE, MPI_DOUBLE, MPI_MAXLOC, 0, MPI_COMM_WORLD );
  MPI_Bcast ( out, 2*TABLE_SIZE, MPI_DOUBLE, 0, MPI_COMM_WORLD );

  /* Coerce ranks to integers */
  for (i=0; i<TABLE_SIZE; i++)
	ranks[i] = out[30+i];
  
  /* Check to see that we got the right answers */
  for (i=0; i<TABLE_SIZE; i++) 
	if (i % size == rank)
	  if (ranks[i] != rank)
		errors++;

  /* Initialize the minloc data */
  for ( i=0; i<TABLE_SIZE; i++ ) a[i] = 0;
  for ( i=rank; i<TABLE_SIZE; i+=size ) a[i] = -(double)rank - 1.0;

  /* Copy data to the "in" buffer */
  for (i=0; i<TABLE_SIZE; i++) 
	in[i] = a[i];

  /* Reduce it! */
  MPI_Allreduce( in, out, 2*TABLE_SIZE, MPI_DOUBLE, MPI_MINLOC, MPI_COMM_WORLD );

  /* Coerce ranks to integers */
  for (i=0; i<TABLE_SIZE; i++)
	ranks[i] = out[30+i];
  
  /* Check to see that we got the right answers */
  for (i=0; i<TABLE_SIZE; i++) 
	if (i % size == rank)
	  if (ranks[i] != rank)
		errors++;

  /* Finish up! */
  MPI_Finalize();
  if (errors)
	printf( "[%d] done with ERRORS(%d)!\n", rank, errors );
  else
	printf( "[%d] SUCCESSFUL!\n", rank );
}
