/**\ --MPE_Log--
*  * mpe_log.h - prototypes for externally callable routines, and a few
*  *             return values
*  *
*  * MPE_Log currently represents some code written by Dr. William
*  * Gropp, stolen from Chameleon's 'blog' logging package and
*  * modified by Ed Karrels, as well as some fresh code written
*  * by Ed Karrels.
*  *
*  * All work funded by Argonne National Laboratory
\**/

#include "mpi.h"

#ifndef _MPE_LOG_H_
#define _MPE_LOG_H_

/* function return values */
#define MPE_Log_OK         0
  /* no problems */
#define MPE_Log_LOCKED_OUT 1
  /* logs are are being worked on, cannot insert any new entries */
#define MPE_Log_NO_MEMORY  2
  /* could not allocate memory for logging data */
#define MPE_Log_FILE_PROB  3
  /* cound not open file for writing out the logged info */
#define MPE_Log_NOT_INITIALIZED 4
  /* logging not initialized */

/* commented out so traditional C compilers don't complain */
#ifdef __STDC__
int MPE_Initlog();		       /* call before calling any other logging functions */
int MPE_Describe_state (int start, int end, char *name, char *color );
							/* create state description lines */
int MPE_Describe_event (int event, char *name);		/* create event description lines */
int MPE_Startlog();					/* set timer to 0 */
int MPE_Log_event (int event, int data, char *string);	/* log one event */
int MPE_Stoplog();					/* stop logging events */
int MPE_Finishlog (char *filename);			/* write out data to a file */

#else
int MPE_Initlog(); 
int MPE_Describe_state ();
int MPE_Describe_event ();
int MPE_Startlog();	
int MPE_Log_event ();
int MPE_Stoplog();	
int MPE_Finishlog();
#endif

#endif

