/*
 *  $Id: chinit.c,v 1.8 1994/04/12 15:58:03 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char SCCSid[] = "%W% %G%";
#endif

/* 
    This file contains the routines that provide the basic information 
    on the device, and initialize it
 */

#include "mpid.h"
#include "dmnx.h"

int __NUMNODES, __MYPROCID;

MPID_INFO *MPID_procinfo = 0;
MPID_H_TYPE MPID_byte_order;
void (*MPID_ErrorHandler)() = MPID_DefaultErrorHandler;

static int DebugSpace = 0;

MPID_SetSpaceDebugFlag( flag )
int flag;
{
DebugSpace = flag;
}

void MPID_NX_myrank( rank )
int *rank;
{
*rank = __MYPROCID;
}

void MPID_NX_mysize( size )
int *size;
{
*size = __NUMNODES;
}

/* 
    In addition, Chameleon processes many command-line arguments 
 */
void MPID_NX_init( argc, argv )
int  *argc;
char **argv;
{
int  i;
char *work;

{__NUMNODES = numnodes();__MYPROCID = mynode();};
/* Turn off the resource monitors */
#if !defined(euih)
/* If we are euih, we can't use SIGALRM; this call sets SIGALRM to 
   SIG_IGN */
;
#endif

#ifdef MPID_HAS_HETERO
MPID_procinfo = (MPID_INFO *)malloc(__NUMNODES * sizeof(MPID_INFO) );
CHKPTR(MPID_procinfo);
for (i=0; i<__NUMNODES; i++) {
    MPID_procinfo[i].byte_order = MPID_H_NONE;
    }
/* Set my byte ordering and convert if necessary.  Eventually, this
   should use xdr */
i = SYGetByteOrder( );
if (i == 1)      MPID_byte_order = MPID_H_LSB;
else if (i == 2) MPID_byte_order = MPID_H_MSB;
else             MPID_byte_order = MPID_H_XDR;
MPID_procinfo[__MYPROCID].byte_order = MPID_byte_order;
/* Everyone uses the same format (MSB) */
if (i == 1) 
    SYByteSwapInt( &MPID_procinfo[__MYPROCID].byte_order, 1 );

/* Get everyone else's */
work = (char *)malloc(__NUMNODES * sizeof(MPID_INFO) );
CHKPTR( work );
/* ASSUMES MPID_INFO is ints */
PIgior( MPID_procinfo, __NUMNODES, work, PSAllProcs );
free(work );

#endif
}

void MPID_NX_Abort( code )
int code;
{
exit(code );
}

void MPID_NX_End()
{
;
if (MPID_procinfo) 
    free(MPID_procinfo );
if (DebugSpace)
    ;

}

void MPID_NX_Node_name( name, len )
char *name;
int  len;
{
sprintf(name,"%d",__MYPROCID);
}

extern double dclock();
double MPID_NX_Wtime()
{
return dclock();
}

/* This returns a value that is correct but not the best value that
   could be returned */
double MPID_NX_Wtick()
{
double t1, t2;
int    cnt;

t1 = MPID_NX_Wtime();
cnt = 1000;
while (cnt-- && (t2 = MPID_NX_Wtime()) <= t1) ;
if (!cnt) return 1.0e6;
return t2 - t1;
}

void MPID_NX_Error_handler( r )
void (*r)();
{
if (r)
    MPID_ErrorHandler = r;
else
    MPID_ErrorHandler = MPID_DefaultErrorHandler;
}

void MPID_DefaultErrorHandler( code, str )
int  code;
char *str;
{
if (str) 
    fprintf( stderr, "%s\n", str );
MPID_NX_Abort( code );
}
