/*
 *  $Id: chsend.c,v 1.9 1994/04/12 15:58:36 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char SCCSid[] = "%W% %G%";
#endif

#include "mpid.h"

/* Non-blocking code untested */
#ifndef PI_NO_NSEND
#define PI_NO_NSEND
#endif

static int DebugFlag = 0;   /* Set to 0 for no debug output */

void MPID_SetSendDebugFlag( f )
int f;
{
DebugFlag = f;
}

/* 
   This file includes the routines to handle the device part of a send
   for Chameleon

   As a reminder, the first element is the device handle, the second is
   the (basically opaque) mpi handle
 */

/*
   This implementation immediately sends the data.

   It takes advantage of being provided with the address of the user-buffer
   in the contiguous case.
 */
int MPID_NX_post_send( dmpi_send_handle ) 
MPIR_SHANDLE *dmpi_send_handle;
{
MPID_SHANDLE *mpid_send_handle;
char        *address;
MPID_PACKET pkt;
int         actual_len, len;
int         dest;

mpid_send_handle = &dmpi_send_handle->dev_shandle;
/* Get the header */
if (mpid_send_handle->start) {
    actual_len = mpid_send_handle->bytes_as_contig;
    }
else {
    DMPI_get_totallen( dmpi_send_handle, &actual_len );
    }
pkt.len	       = len = actual_len;
pkt.context_id = dmpi_send_handle->contextid;
pkt.tag	       = dmpi_send_handle->tag;
pkt.mode       = dmpi_send_handle->mode;
pkt.lrank      = dmpi_send_handle->lrank;
if (pkt.mode == MPIR_MODE_SYNCHRONOUS) 
    MPID_NX_Get_Sync_Id( &pkt.mode, dmpi_send_handle, mpid_send_handle );

dest           = dmpi_send_handle->dest;

if (DebugFlag) {
    printf( "[%d]Starting a send of tag = %d, len = %d, ctx = %d, dest = %d\n",
	    __MYPROCID, pkt.tag, pkt.len, pkt.context_id, dest );
    fflush( stdout );
    }

#ifdef MPID_HAS_HETERO
/* Convert the header into canonical integer order */
if (MPID_procinfo[dest].byte_order != MPID_byte_order) {
    /* Need to swap to receivers order.  We ALWAYS reorder at the
       sender's end */
    SYByteSwapInt( &pkt, 4 );
    }
#endif

#ifdef FOO
/* Eventually, we want to do no copy when the message gets too long */
if (len > MPID_PACKET_SIZE) {
    }
else {
    }
#endif
if (mpid_send_handle->start) {
    memcpy( pkt.buffer, mpid_send_handle->start, 
	    MPID_MIN(MPID_PACKET_SIZE, actual_len ) );
    if (DebugFlag) {
	printf( "[%d]Getting data from mpid->start, first int is %d\n",
	        __MYPROCID, *(int *)&pkt.buffer[0] );
	fflush( stdout );
	}
    }
else {
    DMPI_get_into_contig( dmpi_send_handle, (void *)(pkt.buffer), 
			  MPID_PACKET_SIZE, &actual_len );
    if (DebugFlag) {
	printf( "[%d]Getting data with get_into_contig, first int is %d\n",
	        __MYPROCID, *(int *)&pkt.buffer[0] );
	fflush( stdout );
	}
    }
#if defined(MPID_HAS_HETERO) && 0
if (MPID_procinfo[dest].byte_order != MPID_byte_order) {
    SYByteSwapInt( pkt.buffer, );
    SYByteSwapShort( pkt.buffer, );
    SYByteSwapDouble( pkt.buffer, );
    }
#endif
if (len > MPID_PACKET_SIZE) {
    if (DebugFlag) {
	printf( "[%d]Sending extra-long message...\n", __MYPROCID );
	fflush( stdout );
	}
    /* Send as packet + message */
    _csend(MPID_PT2PT_TAG,&pkt,sizeof(MPID_PACKET),dest,0);
    len			 -= MPID_PACKET_SIZE;
    mpid_send_handle->sid = 0;
    do {
	if (mpid_send_handle->start) {
	    address    = ((char*)mpid_send_handle->start) + MPID_PACKET_SIZE;
	    actual_len = len;
	    }
	else {
	    DMPI_get_from_contig( dmpi_send_handle, (void **)(&address), 
				  -1, &actual_len );
	    }
#if defined(MPID_HAS_HETERO) && 0
	if (MPID_procinfo[dest].byte_order != MPID_byte_order) {
	    SYByteSwapInt( address, );
	    SYByteSwapShort( address, );
	    SYByteSwapDouble( address, );
	    }
	/* Will need to swap back ... */
#endif
#ifndef PI_NO_NSEND
	if (mpid_send_handle->is_non_blocking) {
	    if (mpid_send_handle->sid) {
		/* Not quite correct.  Assumes that only the sid field
		   is used; this is ok as long as the PI_NO_NSEND 
		   wrappers are used... */
		PInwait( MPID_PT2PT2_TAG(__MYPROCID), address, actual_len, 
			 MSG_OTHER, mpid_send_handle->sid );
		}
	    MSG_OTHER=_isend(MPID_PT2PT2_TAG(__MYPROCID),address,actual_len,dest,0);
	    }
	else 
#endif
	    {
	    _csend(MPID_PT2PT2_TAG(__MYPROCID),address,actual_len,dest,0);
	    }
	len -= actual_len;
	} while (len > 0);
#ifndef PI_NO_NSEND
    if ((pkt.mode & 0x7) != MPIR_MODE_SYNCHRONOUS) {
	mpid_send_handle->done = 1;
	DMPI_mark_send_completed( dmpi_send_handle );
	}
#else
    if (mpid_send_handle->is_non_blocking) {
	/* As above, the non-blocking version should be used only when 
	   PI_NO_NSEND not defined */
	mpid_send_handle->buffer = address;
	/* mpid_send_handle->actlen = actual_len */
	}
    else {
	if (dmpi_send_handle->mode != MPIR_MODE_SYNCHRONOUS) {
	    mpid_send_handle->done = YES;
	    DMPI_mark_send_completed( dmpi_send_handle );
	    }
	}
#endif
    }
else {
    /* Always use a blocking send for short messages.
       (May fail with systems that do not provide adequate
       buffering.  These systems should switch to non-blocking sends)
     */
    if (DebugFlag) {
	printf( "[%d]Sending message in a single packet...\n", __MYPROCID );
	fflush( stdout );
	}
    _csend(MPID_PT2PT_TAG,&pkt,actual_len + MPID_HEADER_LEN,dest,0);
    if ((pkt.mode & 0x7) != MPIR_MODE_SYNCHRONOUS) {
	mpid_send_handle->done = YES;
	DMPI_mark_send_completed( dmpi_send_handle );
	}
    }

return MPI_SUCCESS;
}


/*
  Chameleon gets no asynchronous notice that the message has been complete,
  so there is no asynchronous ref to DMPI_mark_send_completed.
 */
int MPID_NX_isend_wait( dmpi_send_handle )
MPIR_SHANDLE *dmpi_send_handle;
{
MPID_SHANDLE *mpid_send_handle;

mpid_send_handle = &dmpi_send_handle->dev_shandle;

/* Wait on the message */
#ifndef PI_NO_NSEND
msgwait(MSG_OTHER);
#endif
if (dmpi_send_handle->mode != MPIR_MODE_SYNCHRONOUS) {
    mpid_send_handle->done = YES;
    DMPI_mark_send_completed( dmpi_send_handle );
    }

return MPI_SUCCESS;
}

int MPID_NX_complete_send( dmpi_send_handle, status ) 
MPIR_SHANDLE *dmpi_send_handle;
MPI_Status   *status;
{
MPID_SHANDLE *mpid_send_handle = &dmpi_send_handle->dev_shandle;

while (!mpid_send_handle->done) {
    (void)MPID_NX_check_incoming( MPID_BLOCKING );
    }
}


