/*
 *  $Id: nxsend.c,v 1.4 1993/12/03 19:39:13 lusk Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char SCCSid[] = "%W% %G%";
#endif

#include "mpid.h"

/* static */ int MPID_NX_debugsend = 0;   /* Set to 0 for no debug output */

/* 
   This file includes the routines to handle the device part of a send
   for NX

   As a reminder, the first element is the device handle, the second is
   the (basically opaque) mpi handle
 */

/*
   This implementation immediately sends the data.

   It takes advantage of being provided with the address of the user-buffer
   in the contiguous case.
 */
int MPID_NX_post_send( dmpi_send_handle ) 
MPIR_SHANDLE *dmpi_send_handle;
{
MPID_SHANDLE *mpid_send_handle;
char        *address;
MPID_PACKET pkt;
int         actual_len, len;
int         dest;

/* Put the message into the queue */
MPIR_enqueue( &posted_sends, dmpi_send_handle, MPIR_QSHANDLE );

mpid_send_handle = &dmpi_send_handle->dev_shandle;
/* Get the header */
if (mpid_send_handle->start) {
    actual_len = mpid_send_handle->bytes_as_contig;
    }
else {
    DMPI_get_totallen( dmpi_send_handle, &actual_len );
    }
pkt.len        = len = actual_len;
pkt.context_id = dmpi_send_handle->contextid;
pkt.tag        = dmpi_send_handle->tag;
dest           = dmpi_send_handle->dest;

if (MPID_NX_debugsend) {
    printf( "[%d]Starting a send of tag = %d, len = %d, dest = %d\n",
	    _NX_me, pkt.tag, pkt.len, dest );
    }
if (mpid_send_handle->start) {
    memcpy( pkt.buffer, mpid_send_handle->start, 
	    MPID_MIN(MPID_PACKET_SIZE, actual_len ) );
    if (MPID_NX_debugsend) {
	printf( "[%d]Getting data from mpid->start, first int is %d\n",
	        _NX_me, *(int *)&pkt.buffer[0] );
	}
    }
else {
    DMPI_get_into_contig( dmpi_send_handle, (void *)(pkt.buffer), 
			  MPID_PACKET_SIZE, &actual_len );
    if (MPID_NX_debugsend) {
	printf( "[%d]Getting data with get_into_contig, first int is %d\n",
	        _NX_me, *(int *)&pkt.buffer[0] );
	}
    }
if (len > MPID_PACKET_SIZE) {
    if (MPID_NX_debugsend) {
	printf( "[%d]Sending extra-long message...\n", _NX_me );
	}
    /* Send as packet + message */
    csend( MPID_PT2PT_TAG, &pkt, sizeof(MPID_PACKET), dest, 0 );
    len -= MPID_PACKET_SIZE;
    mpid_send_handle->sid = -1;
    do {
	if (mpid_send_handle->start) {
	    address = ((char*)mpid_send_handle->start) + MPID_PACKET_SIZE;
	    }
	else {
	    DMPI_get_from_contig( dmpi_send_handle, (void **)(&address), 
				  -1, &actual_len );
	    }
	    csend( MPID_PT2PT2_TAG(_NX_me), address, actual_len, dest, 0 );
	len -= actual_len;
	} while (len > 0);
/* XXX blocking behavior */
#if 0
	mpid_send_handle->buffer = address;
	/* mpid_send_handle->actlen = actual_len */
#else
	mpid_send_handle->done = YES;
	DMPI_mark_send_completed( dmpi_send_handle );
#endif
    }
else {
    /* Always use a blocking send for short messages.
       (May fail with systems that do not provide adequate
       buffering.  These systems should switch to non-blocking sends)
     */
    if (MPID_NX_debugsend) {
	printf( "[%d]Sending message in a single packet...\n", _NX_me );
	}
    csend( MPID_PT2PT_TAG, &pkt, actual_len + MPID_HEADER_LEN, dest, 0 );
    mpid_send_handle->done = YES;
    DMPI_mark_send_completed( dmpi_send_handle );
    }

return MPI_OK;
}


/*
  We use no asynchronous notice that the message has been complete,
  so there is no asynchronous ref to DMPI_mark_send_completed.
 */
int MPID_NX_isend_wait( dmpi_send_handle )
MPIR_SHANDLE *dmpi_send_handle;
{
MPID_SHANDLE *mpid_send_handle;

mpid_send_handle = &dmpi_send_handle->dev_shandle;

mpid_send_handle->done = YES;
DMPI_mark_send_completed( dmpi_send_handle );

return MPI_OK;
}

int MPID_NX_complete_send( dmpi_send_handle, status ) 
MPIR_SHANDLE *dmpi_send_handle;
MPI_Status   *status;
{
MPID_SHANDLE *mpid_send_handle = &dmpi_send_handle->dev_shandle;

while (!mpid_send_handle->done) {
    (void)MPID_NX_check_incoming( MPID_BLOCKING );
    }

if (MPID_NX_debugsend) {
    printf( "[%d] About to dequeue a message\n", _NX_me );
    }
MPIR_dequeue( &posted_sends, dmpi_send_handle );
}


