/*
 *  $Id: allreduce.c,v 1.13 1994/04/13 21:57:08 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: allreduce.c,v 1.13 1994/04/13 21:57:08 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_Allreduce - Combine values from all processes and distribute the result
                back to all processes

Input Parameters:
. sendbuf - starting address of send buffer (choice) 
. count - number of elements in send buffer (integer) 
. datatype - data type of elements of send buffer (handle) 
. op - operation (handle) 
. comm - communicator (handle) 

Output Parameter:
. recvbuf - starting address of receive buffer (choice) 
@*/
int MPI_Allreduce ( sendbuf, recvbuf, count, datatype, op, comm )
void             *sendbuf;
void             *recvbuf;
int               count;
MPI_Datatype      datatype;
MPI_Op            op;
MPI_Comm          comm;
{
  int errno = MPI_SUCCESS;
  int flag;

  /* Check for invalid arguments */
  if ( ( (comm == MPI_COMM_NULL) && (errno = MPI_ERR_COMM) )   ||
       ( (op == MPI_OP_NULL)     && (errno = MPI_ERR_OP) )     ||
	   ( (sendbuf == recvbuf)    && (errno = MPI_ERR_BUFFER) ) )
    return MPIR_ERROR(comm, errno, "Error in MPI_ALLREDUCE" );
  
  /* Check for intra-communicator */
  MPI_Comm_test_inter ( comm, &flag );
  if (flag) 
    return MPIR_ERROR(comm, MPI_ERR_COMM,
					  "Inter-communicator invalid in MPI_ALLREDUCE");

  /* Reduce to 0, then bcast */
  MPI_Reduce ( sendbuf, recvbuf, count, datatype, op, 0, comm );
  MPI_Bcast  ( recvbuf, count, datatype, 0, comm );

  return (errno);
}
