/*
 *  $Id: attr_delval.c,v 1.9 1994/04/07 08:55:54 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

#ifndef MPIR_TRUE
#define MPIR_TRUE  1
#define MPIR_FALSE 0
#endif

/*@

MPI_Attr_delete - deletes attribute value associated with a key

Input Parameters:
. comm - communicator to which attribute is attached (handle) 
. keyval - The key value of the deleted attribute (integer) 

@*/
int MPI_Attr_delete ( comm, keyval )
MPI_Comm comm;
int keyval;
{
  MPIR_HBT_node *attr;
  MPIR_Attr_key *attr_key = (MPIR_Attr_key *)keyval;
  int            status   = MPI_SUCCESS;

  if ( (comm != MPI_COMM_NULL) && (keyval != 0) ) {
    if (comm == MPI_COMM_WORLD && attr_key->permanent) 
	return MPIR_ERROR( comm, MPI_ERR_PERM_KEY, 
			   "Error in MPI_ATTR_DELETE" );
    if ( attr_key->delete_fn != (int(*)())0 ) 
      status = attr_key->delete_fn( comm, keyval, attr_key->extra_state );
    MPIR_HBT_delete(comm->attr_cache, keyval, &attr);
    if ( attr != (MPIR_HBT_node *)0 ) 
      (void) MPIR_HBT_free_node ( attr );
  }
  return(status);
}
