/*
 *  $Id: attr_getkey.c,v 1.6 1994/04/01 23:31:44 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>
#include "mpisys.h"
/*@

MPI_Keyval_create - generates a new attribute key

Input Parameters:
. copy_fn - Copy callback function for keyval 
. delete_fn - Delete callback function for keyval 
. extra_state - Extra state for callback functions 

Output Parameter:
. keyval - key value for future access (integer) 
@*/
int MPI_Keyval_create ( copy_fn, delete_fn, keyval, extra_state )
MPI_Copy_function   *copy_fn;
MPI_Delete_function *delete_fn;
int                 *keyval;
void                *extra_state;
{
  MPIR_Attr_key *new_key = NEW(MPIR_Attr_key);
  if (!new_key) {
      return MPIR_ERROR( (MPI_Comm)0, MPI_ERR_EXHAUSTED, 
			 "Out of space in MPI_Keyval_create" );
      }
  /* NOTE: THIS WON'T WORK ON ALL SYSTEMS: */
  (*keyval)              = (int)new_key;
  /* SEE ALSO THE CODE IN ENV/INIT.C; IT RELIES ON USEING KEY AS THE
     POINTER TO SET THE PERMANENT FIELD */
  new_key->copy_fn       = copy_fn;
  new_key->delete_fn     = delete_fn;
  new_key->ref_count     = 1;
  new_key->extra_state   = extra_state;
  new_key->permanent     = 0;
  return (MPI_SUCCESS);
}

