/*
 *  $Id: comm_util.c,v 1.12 1994/04/13 21:43:49 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */
#include <mpi.h>
#include "mpisys.h"

/*

MPIR_Comm_make_coll - make a hidden collective communicator
                      from an inter- or intra-communicator assuming
		      that an appropriate number of contexts
		      have been allocated.  An inter-communicator
		      collective can only be made from another
		      inter-communicator.

*/
int MPIR_Comm_make_coll ( comm, comm_type )
MPI_Comm       comm;
MPIR_COMM_TYPE comm_type;
{
  MPI_Comm new_comm         = NEW(MPIR_COMMUNICATOR);
  new_comm->comm_type       = comm_type;
  new_comm->attr_cache      = comm->attr_cache;
  new_comm->comm_cache      = comm->comm_cache;

  /* If I'm making an intra-communicator, then the recieve context
     is incremented (the recv_context is a locally allocated context
     for both inter- and intra-comms) by 1 and assigned to the
     send and recv contexts of the new intra-communicator.  All groups
     are the same and are equivalent to the "local_group" of the
     old communicator. */
  if (comm_type == MPIR_INTRA) {
      new_comm->send_context= new_comm->recv_context= comm->recv_context + 1;
      new_comm->group       = new_comm->local_group = comm->local_group;
  }
  /* If I'm making an inter-communicator, then the send and recv contexts
     are probably different and incremented by 1.  The local and remote
     groups are different. */
  else {
      new_comm->send_context= comm->send_context + 1;
      new_comm->recv_context= comm->recv_context + 1;
      new_comm->group       = comm->group;
      new_comm->local_group = comm->local_group;
  }
  new_comm->error_handler   = comm->error_handler;
  new_comm->comm_coll       = new_comm;  /* a circular reference to myself */
  comm->comm_coll           = new_comm;

  return(MPI_SUCCESS);
}


/*+

MPIR_Comm_N2_prev - retrieve greatest power of two < size of Comm.

+*/
int MPIR_Comm_N2_prev ( comm, N2_prev )
MPI_Comm comm;
int              *N2_prev;
{
  (*N2_prev) = comm->group->N2_prev;
  return (MPI_SUCCESS);
}
