/*
 *  $Id: group_rank.c,v 1.6 1994/04/07 08:55:54 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Group_rank - Return the rank of this process in the given group

Input Parameters:
. group - group (handle) 
Output Parameter:
. rank - rank of the calling process in group, or   MPI_UNDEFINED  if the 
process is not a member (integer) 

@*/
int MPI_Group_rank ( group, rank )
MPI_Group group;
int *rank;
{
  int errno = MPI_SUCCESS;

  /* Check for invalid arguments */
  if ( ( (group == MPI_GROUP_NULL) && (errno = MPI_ERR_GROUP) ) ||
       ( (rank  != (int *)0)       && (errno = MPI_ERR_ARG) )   )
	return MPIR_ERROR(MPI_COMM_NULL,errno,"Error in MPI_GROUP_RANK");

  /* Get the rank of the group */
  (*rank) = group->local_rank;

  return (errno);
}
