/*
 *  $Id: error_string.c,v 1.2 1994/04/12 20:26:36 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: error_string.c,v 1.2 1994/04/12 20:26:36 doss Exp $";
#endif /* lint */
#include "mpi.h"
#include "mpisys.h"

/*@
   MPI_Error_string - Return a string for a given error code

Input Parameters:
. errorcode - Error code returned by an MPI routine 
. len - Length of string 

Output Parameter:
. string - Text that corresponds to the errorcode 
@*/
int MPI_Error_string( errorcode, len, string )
int  errorcode, len;
char *string;
{
switch (errorcode & 0xff) {
    case MPI_SUCCESS:
        strncpy( string, "No error", len );
	break;
    case MPI_EXAUSTED: 
        strncpy( string, "Out of internal memory", len );
        break;
    case MPI_INVALID_TAG:
	strncpy( string, "Invalid message tag", len );
	break;
    case MPI_ERR_COMM_NULL:
	strncpy( string, 
		 "NULL communicator argument passed to function", len );
	break;
    case MPI_ERR_COMM_INTRA:
	strncpy( string,
		 "Intracommunicator is not allowed in function", len );
	break;
    case MPI_ERR_COMM_INTER:
	strncpy( string,
		 "Intercommunicator is not allowed in function", len );
	break;
    case MPI_ERR_ARG:
	strncpy( string, "Invalid argument", len );
	break;
    case MPI_ERR_BUFFER:
	strncpy( string, "Invalid buffer pointer", len );
	break;
    case MPI_ERR_COUNT:
        strncpy( string, "Invalid count argument", len );
	break;
    case MPI_ERR_TYPE:
        strncpy( string, "Invalid datatype argument", len );
	break;
    case MPI_ERR_ROOT:
	strncpy( string, "Invalid root", len );
	break;
    case MPI_ERR_OP:
	strncpy( string, "Invalid operation", len );
	break;
    case MPI_ERR_ERRORCODE:
	strncpy( string, "Invalid error code", len );
	break;
    case MPI_ERR_GROUP_NULL:
	strncpy( string, "Null group passed to function", len );
	break;
    case MPI_ERR_RANK:
        strncpy( string, "Invalid rank", len );
	break;
    case MPI_ERR_TOPOLOGY:
	strncpy( string, "Invalid topology", len );
	break;
    case MPI_ERR_DIMS:
        strncpy( string, "Illegal dimension argument", len );
	break;
    case MPI_ERR_NULL:
        strncpy( string, "Null parameter", len );
	break;
    case MPI_ERR_UNKNOWN:
	strncpy( string, "Unknown error", len );
	break;
    case MPI_ERR_INTERN:
	strncpy( string, "Internal MPI error!", len );
	break;
    case MPI_ERR_BAD_ARGS:
	strncpy( string, "Invalid arguments to MPI routine", len );
	break;
    case MPI_ERR_INIT:
	strncpy( string, "Can not call MPI_INIT twice!", len );
	break;
    case MPI_ERR_PERM_KEY:
	strncpy( string, "Can not free permanent attribute key", len );
	break;
    case MPI_ERR_BUFFER_EXISTS:
	strncpy( string,
		"Can not attach buffer when a buffer already exists", len );
	break;
    case MPI_ERR_TRUNCATE:
	strncpy( string, "Message truncated", len );
	break;
    case MPI_ERR_LIMIT:
	strncpy( string, "System resource limit exceeded", len );
	break;
    case MPI_ERR_COMM:
	strncpy( string, "Invalid communicator", len );
	break;
    case MPI_ERR_OTHER:
	strncpy( string, "Unclassified error", len );
	break;
    default:
	strncpy( string, "Unexpected error value!", len );
        break;
    }
return MPI_SUCCESS;
}
