/*
 *  $Id: finalize.c,v 1.11 1994/04/15 23:15:50 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include "mpi.h"
#include "mpisys.h"

extern int MPIR_Print_queues;
#define DBG(a)

/*@
   MPI_Finalize - Terminate MPI execution environment
@*/
int MPI_Finalize()
{
    DBG(fprintf( stderr, "Entering system finalize\n" ); fflush(stderr);)
    /*  Dump final status of queues */
    if (MPIR_Print_queues) {
	int i, np, rank;
	MPI_Comm_rank( MPI_COMM_WORLD, &rank );
	MPI_Comm_size( MPI_COMM_WORLD, &np );
	for (i=0 ; i<np; i++) {
	    MPI_Barrier( MPI_COMM_WORLD );
	    if (i == rank) {
		printf("[%d]Dumping recv queue...\n", rank );
		MPIR_dump_queue( &MPIR_posted_recvs );
		fflush( stdout );
		}
	    }
	for (i=0 ; i<np; i++) {
	    MPI_Barrier( MPI_COMM_WORLD );
	    if (i == rank) {
		printf("[%d]Dumping unexpected queue...\n", rank );
		MPIR_dump_queue( &MPIR_unexpected_recvs );
		fflush( stdout );
		}
	    }
	}

    DBG(fprintf( stderr, "About to free operators\n" ); fflush( stderr );)
    MPI_Op_free( &MPI_MAX );
    MPI_Op_free( &MPI_MIN );
    MPI_Op_free( &MPI_SUM );
    MPI_Op_free( &MPI_PROD );
    MPI_Op_free( &MPI_LAND );
    MPI_Op_free( &MPI_BAND );
    MPI_Op_free( &MPI_LOR );
    MPI_Op_free( &MPI_BOR );
    MPI_Op_free( &MPI_LXOR );
    MPI_Op_free( &MPI_BXOR );
#ifdef FOO
    MPI_Op_free( &MPI_MAXLOC );
    MPI_Op_free( &MPI_MINLOC );
#endif

    /* Free allocated space */
    DBG(fprintf( stderr, "About to free dtes\n" ); fflush( stderr );)
    SBfree( MPIR_dtes, MPIR_int_dte );
    SBfree( MPIR_dtes, MPIR_float_dte );
    SBfree( MPIR_dtes, MPIR_double_dte );
    SBfree( MPIR_dtes, MPIR_long_dte );
    SBfree( MPIR_dtes, MPIR_complex_dte );
    SBfree( MPIR_dtes, MPIR_dcomplex_dte );
    SBfree( MPIR_dtes, MPIR_short_dte );
    SBfree( MPIR_dtes, MPIR_char_dte );
    SBfree( MPIR_dtes, MPIR_byte_dte );
    SBfree( MPIR_dtes, MPIR_uchar_dte );
    SBfree( MPIR_dtes, MPIR_ushort_dte );
    SBfree( MPIR_dtes, MPIR_ulong_dte );
    SBfree( MPIR_dtes, MPIR_uint_dte );
    SBfree( MPIR_dtes, MPIR_packed_dte );
    SBfree( MPIR_dtes, MPIR_ub_dte );
    SBfree( MPIR_dtes, MPIR_lb_dte );
#if defined(__STDC__) && !defined(MPI_rs6000)
    SBfree( MPIR_dtes, MPIR_longdouble_dte );
#endif

    DBG(fprintf( stderr, "About to free COMM_WORLD\n" ); fflush( stderr );)

    MPIR_HBT_free_tree ( MPI_COMM_WORLD->attr_cache );
    MPIR_FreeGroup( MPI_COMM_WORLD->group );
    FREE( MPI_COMM_WORLD );
    /* We need to set MPI_COMM_WORLD to zero to insure that a subsequent
       init generates the correct error message */
    MPI_COMM_WORLD = 0;

    DBG(fprintf( stderr, "About to free COMM_SELF\n" ); fflush( stderr );)

    MPIR_HBT_free_tree ( MPI_COMM_SELF->attr_cache );
    MPIR_FreeGroup( MPI_COMM_SELF->group );
    FREE( MPI_COMM_SELF );

    DBG(fprintf( stderr, "About to free GROUP_EMPTY\n" ); fflush( stderr );)

    MPIR_FreeGroup( MPI_GROUP_EMPTY );

    /* Tell device that we are done.  We place this here to allow
       the device to tell us about any memory leaks, since SB... will
       free the storage even if it has not been deallocated by SBfree. 
     */
    DBG(fprintf( stderr, "About to close device\n" ); fflush( stderr );)

    MPID_END();

    DBG(fprintf( stderr, "About to free SBstuff\n" ); fflush( stderr );)

    SBdestroy( MPIR_dtes );
    SBdestroy( MPIR_qels );
    SBdestroy( MPIR_fdtels );
    SBdestroy( MPIR_shandles );
    SBdestroy( MPIR_rhandles );

    SBdestroy( MPIR_hbts );
    SBdestroy( MPIR_hbt_els );

    /* barrier */
    return MPI_SUCCESS;
}





