/*
 *  $Id: create_recv.c,v 1.2 1994/04/12 16:00:41 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include "mpi.h"
#include "mpisys.h"

/*@
    MPI_Recv_init - Create a handle for a receive

Input Parameters:
. count - number of elements received (integer) 
. datatype - type of each element (handle) 
. dest - rank of source or MPI_ANY_SOURCE (integer) 
. tag - message tag or MPI_ANY_TAG (integer) 
. comm - communicator (handle) 
Output Parameters:
. buf - initial address of receive buffer (choice) 
. request - communication request (handle) 
@*/
int MPI_Recv_init( buf, count, datatype, source, tag, comm, request )
void         *buf;
int          count;
MPI_Request  *request;
MPI_Datatype datatype;
int          source;
int          tag;
MPI_Comm     comm;
{

    MPIR_HANDLE *handleptr;

    *request			   = (MPI_Request) SBalloc( MPIR_rhandles );
    handleptr			   = *request;
    handleptr->type		   = MPIR_RECV;
    if (source == MPI_ANY_SOURCE)
	handleptr->rhandle.source  = source;
    else
	handleptr->rhandle.source  = comm->group->lrank_to_grank[source];
    handleptr->rhandle.tag	   = tag;
    handleptr->rhandle.contextid   = comm->recv_context;
    handleptr->rhandle.datatype	   = datatype;
    handleptr->rhandle.bufadd      = buf;
    handleptr->rhandle.count       = count;

    handleptr->rhandle.completed   = NO;
    MPID_alloc_recv_handle(&((handleptr)->rhandle.dev_rhandle));

    return MPI_SUCCESS;
}
