/*
 *  $Id: sendrecv_rep.c,v 1.2 1994/04/01 18:02:51 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: sendrecv_rep.c,v 1.2 1994/04/01 18:02:51 gropp Exp $";
#endif /* lint */

#include "mpi.h"

/*@
    MPI_Sendrecv_replace - Send and receive using a single buffer

Input Parameters:
. count - number of elements in send and receive buffer (integer) 
. datatype - type of elements in send and receive buffer (handle) 
. dest - rank of destination (integer) 
. sendtag - send message tag (integer) 
. source - rank of source (integer) 
. recvtag - receive message tag (integer) 
. comm - communicator (handle) 

Output Parameters:
. buf - initial address of send and receive buffer (choice) 
. status - status object (Status) 
@*/
int MPI_Sendrecv_replace( buf, count, datatype, dest, sendtag, 
			  source, recvtag, comm, status )
void         *buf;
int           count, dest, sendtag, source, recvtag;
MPI_Datatype  datatype;
MPI_Comm      comm;
MPI_Status   *status;
{
    int               errno = MPI_SUCCESS;
    void              *rbuf;
    MPI_Status        status_array[2];
    MPI_Request  req[2];

    /* Check for invalid arguments */
    if ( (comm == (MPI_Comm)0) )
      return MPIR_ERROR( comm, MPI_ERR_COMM_NULL, 
			 "Error in MPI_SENDRECV_REPL" );

    /* We really need to get the length in bytes FIRST, post the Irecv,
       THEN the Isend... */

    if (errno = MPI_Isend ( buf,  count, datatype, dest,   
			    sendtag, comm, &req[0] )) return errno;
    if (req[0]->shandle.buflen > 0) {
	rbuf = (void *)MALLOC( (unsigned)(req[0]->shandle.buflen) );
	if (!rbuf) {
	    return MPIR_ERROR( comm, MPI_ERR_EXHAUSTED, 
			      "Error in MPI_SENDRECV_REPL" );
	    }
	}
    else
	rbuf = (void *)0;

    if (errno = MPI_Irecv ( rbuf, count, datatype, source, 
			    recvtag, comm, &req[1] )) return errno;
    errno = MPI_Waitall ( 2, req, status_array );
    if (rbuf) {
	memcpy( buf, rbuf, req[1]->rhandle.buflen );
	FREE( rbuf );
	}

    (*status) = status_array[1];
    return errno;
}
