/*
 *  $Id: start.c,v 1.8 1994/04/06 18:56:01 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: start.c,v 1.8 1994/04/06 18:56:01 gropp Exp $";
#endif /* lint */

#include "mpi.h"
#include "mpisys.h"

/* NOTES 
   We mark all sends and receives as non-blocking because that is safe here;
   unfortunately, we don't have enough information in the current send/recv
   handle to determine if we are blocking or not.
 */

/*@
    MPI_Start - Initiate a communication with a persistent request handle

Input Parameter:
. request - communication request (handle) 
@*/
int MPI_Start( request )
MPI_Request *request;
{
    int is_available;
    int errno = MPI_SUCCESS;

    /* See also send.c */
    if ((*request)->type == MPIR_SEND)
    {
	if ((*request)->shandle.datatype->is_contig) {
	    (*request)->shandle.dev_shandle.start = (*request)->shandle.bufadd;
            (*request)->shandle.dev_shandle.bytes_as_contig =
	     (*request)->shandle.count * (*request)->shandle.datatype->extent;
	    }
	/* device will post the send */
	MPID_post_send( &((*request)->shandle) );
    }
    else if ((*request)->type == MPIR_RECV)
    {
	if ((*request)->rhandle.datatype->is_contig) {
	    (*request)->rhandle.dev_rhandle.start = (*request)->rhandle.bufadd;
            (*request)->rhandle.dev_rhandle.bytes_as_contig =
	     (*request)->rhandle.count * (*request)->rhandle.datatype->extent;
	    }
        /* device will handle queueing of MPIR recv handle in posted-recv 
	   queue         
	 */
        MPID_set_recv_is_nonblocking( &((*request)->rhandle.dev_rhandle), 1 );
	MPID_post_recv( &((*request)->rhandle), &(is_available) );
    }
    else
	errno = MPIR_ERROR(MPI_COMM_WORLD,MPI_ERR_INTERN,
		   "bad request type in MPI_START");
    return errno;
}


