/*
 *  $Id: testsome.c,v 1.4 1994/04/13 13:25:19 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: testsome.c,v 1.4 1994/04/13 13:25:19 gropp Exp $";
#endif /* lint */
#include "mpi.h"
#include "mpisys.h"

/*@
    MPI_Testsome - Test for some given communications to complete

Input Parameters:
. incount - length of array_of_requests (integer) 
. array_of_requests - array of requests (array of handles) 

Output Parameters:
. outcount - number of completed requests (integer) 
. array_of_indices - array of indices of operations that 
completed (array of integers) 
. array_of_statuses - array of status objects for 
    operations that completed (array of Status) 
@*/
int MPI_Testsome( incount, array_of_requests, outcount, array_of_indices, 
		  array_of_statuses )
int         incount, *outcount, array_of_indices[];
MPI_Request array_of_requests[];
MPI_Status  array_of_statuses[];
{
    int i;
    MPIR_BOOL completed;
    int nfound = 0;

    /* NOTE:
       This implementation will not work correctly if the device requires
       messages to be received in some particular order.  In that case, 
       this routine needs to try and complete the messages in ANY order.
       
       The same is true for testall.c .
     */
    MPID_check_device( MPID_NOTBLOCKING );
    for (i = 0; i < incount; i++)
	{
	/* Skip over null handles.  We need this for handles generated
	   when MPI_PROC_NULL is the source or destination of an 
	   operation */
	if (!array_of_requests[i]) continue;
	
	/* This should really use complete_send and complete_recv */
	
	if (array_of_requests[i]->chandle.completed) {
	    
	    array_of_indices[nfound++] = i;
	    if ( array_of_requests[i]->type == MPIR_RECV )
		{
		MPID_complete_recv( &array_of_requests[i]->rhandle );
		
		array_of_statuses[i].MPI_SOURCE = 
		    array_of_requests[i]->rhandle.source;
		array_of_statuses[i].MPI_TAG    = 
		    array_of_requests[i]->rhandle.tag;
		array_of_statuses[i].count  = 
		    array_of_requests[i]->rhandle.actcount;  
		array_of_statuses[i].count_in_bytes =
		    array_of_requests[i]->rhandle.totallen;
		
		}
	    else 
		MPID_complete_send( &array_of_requests[i]->shandle, 
				   &(array_of_statuses[i]) );
	    
	    /* Spec requires, but... */
	    MPI_Request_free( &array_of_requests[i] ); 
	    array_of_requests[i]    = NULL;
	    }
	}
    *outcount = nfound;
    return MPI_SUCCESS;
}
