/*
 *  $Id: type_free.c,v 1.1 1994/04/18 15:47:07 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include "mpi.h"
#include "mpisys.h"

#ifndef MPIR_TRUE
#define MPIR_TRUE  1
#define MPIR_FALSE 0
#endif

/*@
    MPI_Type_free - Frees the datatype
@*/
int MPI_Type_free ( datatype )
MPI_Datatype *datatype;
{
  int errno = MPI_SUCCESS;

  /* Check for bad arguments */
  if ( ( (datatype    == (MPI_Datatype *)0) && (errno = MPI_ERR_ARG) ) ||
	   ( ((*datatype) == MPI_DATATYPE_NULL) && (errno = MPI_ERR_TYPE) ) )
	return MPIR_ERROR( MPI_COMM_NULL, errno,
					  "Error in MPI_TYPE_FREE" );

  /* Don't free basic types */
  if ((*datatype)->basic == MPIR_TRUE)
	return (MPI_SUCCESS);

  /* Free datatype */
  if ( (*datatype)->ref_count <= 1 ) {

	/* Free malloc'd memory for various datatypes */
	if ( ((*datatype)->dte_type == MPIR_INDEXED)  ||
		 ((*datatype)->dte_type == MPIR_HINDEXED) || 
		 ((*datatype)->dte_type == MPIR_STRUCT)   ) {
	  FREE ( (*datatype)->indices );
	  FREE ( (*datatype)->blocklens );
	}

	/* Free the old_type if not a struct */
	if ((*datatype)->dte_type != MPIR_STRUCT)
	  MPI_Type_free ( &((*datatype)->old_type) );
	
	/* Free the old_types of a struct */
	if ((*datatype)->dte_type == MPIR_STRUCT) {
	  int i;

	  /* Decrease the reference count */
	  for (i=0; i<(*datatype)->count; i++)
		MPI_Type_free ( &((*datatype)->old_types[i]) );

	  /* Free the malloc'd memory */
	  FREE ( (*datatype)->old_types );
	}

	/* Free the datatype structure */
	SBfree ( MPIR_dtes, (*datatype) );
  }
  else 
	(*datatype)->ref_count--;

  (*datatype) = MPI_DATATYPE_NULL;
  return (errno);
}
