/*
 *  $Id: get_cartdim.c,v 1.7 1994/03/29 22:25:01 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Cartdim_get - Retrieve cart topology information associated with a 
                  communicator

Input Parameter:
. comm - communicator with cartesian structure (handle) 

Output Parameter:
. ndims - number of dimensions of the cartesian structure (integer) 

@*/
int MPI_Cartdim_get ( comm, ndims )
MPI_Comm  comm;
int              *ndims;
{
  /* Check for valid arguments */
  if ( comm == (MPI_Comm)0 ) 
    return MPIR_ERROR( comm, MPI_ERR_COMM_NULL, "Error in MPI_CARTDIM_GET" );

  /* Set dims */
  if ( ndims != (int *)0 )
    if ( comm->topology.type == MPI_CART )
      (*ndims) = comm->topology.tag.cart.ndims;
    else
      (*ndims) = MPI_UNDEFINED;

  return (MPI_SUCCESS);
}
