/*
 *  $Id: graph_map.c,v 1.2 1994/04/12 20:12:58 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Graph_map - Map process to graph topology information

Input Parameters:
. comm - input communicator (handle) 
. nnodes - number of graph nodes (integer) 
. index - integer array specifying the graph structure, see  MPI_GRAPH_CREATE 
. edges - integer array specifying the graph structure 
Output Parameter:
. newrank - reordered rank of the calling process; MPI_UNDEFINED if the calling process does not belong to graph (integer) 
 

@*/
int MPI_Graph_map ( comm_old, nnodes, index, edges, newrank )
MPI_Comm comm_old;
int      nnodes;
int     *index;
int     *edges;
int     *newrank;
{
  int i;
  int nranks;
  int rank;
  int errno = MPI_SUCCESS;

  /* Check for valid arguments */
  if (((comm_old == MPI_COMM_NULL) && (errno = MPI_ERR_COMM)) ||
      ((nnodes   <  1)             && (errno = MPI_ERR_ARG))  ||
      ((newrank  == (int *)0)      && (errno = MPI_ERR_ARG))  ||
      ((index    == (int *)0)      && (errno = MPI_ERR_ARG))  ||
      ((edges    == (int *)0)      && (errno = MPI_ERR_ARG))   )
    return MPIR_ERROR( comm_old, errno, "Error in MPI_GRAPH_MAP" );
  
  /* Am I in this topology? */
  MPI_Comm_rank ( comm_old, &rank );
  if ( rank < nnodes )
    (*newrank) = rank;
  else
    (*newrank) = MPI_UNDEFINED;

  return (errno);
}
