/*
 *  $Id: graph_nbr.c,v 1.9 1994/04/12 20:12:58 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

#ifndef MPIR_MIN
#define MPIR_MIN(a,b) ((a)>(b)?(b):(a))
#endif

/*@

MPI_Graph_neighbors - Returns the neighbors of a node associated 
                      with a graph topology

Input Parameters:
. comm - communicator with graph topology (handle) 
. rank - rank of process in group of comm (integer) 
O
utput Parameters:
. maxneighbors - size of array neighbors (integer) 
. neighbors - ranks of processes that are neighbors to specified process (array of integer) 

@*/
int MPI_Graph_neighbors ( comm, rank, maxneighbors, neighbors )
MPI_Comm  comm;
int       rank;
int      *maxneighbors;
int      *neighbors;
{
  int i, begin, end;
  int errno = MPI_SUCCESS;

  /* Check for valid arguments */
  if (((comm                == MPI_COMM_NULL) && (errno = MPI_ERR_COMM)) ||
      ((comm->topology.type != MPI_GRAPH)     && (errno = MPI_ERR_TOPOLOGY))  ||
      ((rank                <  0)             && (errno = MPI_ERR_RANK))      ||
      ((rank >= (comm->topology.tag.graph.nnodes))&&(errno = MPI_ERR_RANK)) ||
      ((neighbors           == (int *)0)      && (errno = MPI_ERR_ARG))       )
    return MPIR_ERROR( comm, errno, "Error in MPI_GRAPH_NEIGHBORS" );

  /* Get neighbors */
  if ( rank == 0 ) 
    begin = 0;
  else
    begin = comm->topology.tag.graph.index[rank-1];
  end = comm->topology.tag.graph.index[rank];
  for ( i=begin; i<end; i++ )
    neighbors[i-begin] = comm->topology.tag.graph.edges[i];
  (*maxneighbors) = end - begin;
  return (errno);
}
