/*
 *  $Id: graphdims_get.c,v 1.2 1994/04/12 20:12:58 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>

/*@

MPI_Graphdims_get - retrieve graph topology information associated with a 
                    communicator

Input Parameters:
. comm - communicator for group with graph structure (handle) 
Output Parameter:
. nnodes - number of nodes in graph (integer) 
. nedges - number of edges in graph (integer) 

@*/
int MPI_Graphdims_get ( comm, nnodes, nedges )
MPI_Comm  comm;
int              *nnodes;
int              *nedges;
{
  /* Check for valid arguments */
  if ( comm == MPI_COMM_NULL )
    return MPIR_ERROR( MPI_COMM_NULL, MPI_ERR_COMM, 
		      "Error in MPI_GRAPHDIMS_GET" );

  /* Set nnodes */
  if ( nnodes != (int *)0 )
    if ( comm->topology.type == MPI_GRAPH )
      (*nnodes) = comm->topology.tag.graph.nnodes;
    else
      (*nnodes) = MPI_UNDEFINED;

  /* Set nedges */
  if ( nedges != (int *)0 ) 
    if ( comm->topology.type == MPI_GRAPH )
      (*nedges) = comm->topology.tag.graph.nedges;
    else
      (*nedges) = MPI_UNDEFINED;

  return (MPI_SUCCESS);
}
