/*
 *  $Id: send.c,v 1.11 1994/04/18 21:33:45 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: send.c,v 1.11 1994/04/18 21:33:45 gropp Exp $";
#endif /* lint */

#include "mpi.h"

/*@
    MPI_Send - Basic send

Input Parameters:
. buf - initial address of send buffer (choice) 
. count - number of elements in send buffer (nonnegative integer) 
. datatype - datatype of each send buffer element (handle) 
. dest - rank of destination (integer) 
. tag - message tag (integer) 
. comm - communicator (handle) 

@*/
int MPI_Send( buf, count, datatype, dest, tag, comm )
void             *buf;
int              count, dest, tag;
MPI_Datatype     datatype;
MPI_Comm         comm;
{
    int          err;
    MPIR_SHANDLE shandle;
    MPI_Request  request;
    MPI_Status   status;

    /* 
       Because this is a very common routine, we show how it can be
       optimized to be run "inline"; In addition, this lets us exploit
       features in the ADI to simplify the execution of blocking send 
       calls.
     */
    if (dest != MPI_PROC_NULL)
    {
        request = (MPIR_HANDLE *)&shandle;
        MPIR_Send_init( buf, count, datatype, dest, tag, comm, request, 
		        MPIR_MODE_STANDARD, 0 );
	if (shandle.datatype->is_contig) {
	    shandle.dev_shandle.start		= buf;
            shandle.dev_shandle.bytes_as_contig	= count * datatype->extent;
	    shandle.bufpos                      = 0;
	    }
#ifdef MPID_PACK_IN_ADVANCE
	else {
	    if (err = 
		MPIR_PackMessage( buf, count, datatype, dest, request )) {
		MPIR_ERROR( comm, err, "Could not pack message in MPI_SEND" );
		}
	    }
#else
	else {
	    shandle.dev_shandle.start = 0;
	    }
#endif
	MPID_post_send( &shandle );
        MPID_complete_send( &shandle, &status );
#ifdef MPID_PACK_IN_ADVANCE
	if (shandle.bufpos && (err = MPIR_EndPackMessage( request ))){
	    MPIR_ERROR( comm, err, "Could not pack message in MPI_SEND" );
	    }
#endif
    }
    return MPI_SUCCESS;
}

